% This is the main program.  Executing it in MATLAB will recreate all
% numerical results and figures in the theoretical section of 
% Koijen-Philipson-Uhlig, "Financial Health Economics".
% To see all resulting graphs and tables, compile all_graphs.tex with LaTeX
% (note that the initial tables there have been copied from the MATLAB
% output of this program).
%
% FIGURES:
% The program stores the figures as .eps files in the current directory.
% The graphs used in the paper are
% model_Ralph.eps, counter_Ralph.eps, counter_Ralph_longrun.eps,
% However, the program generates more graphs, allowing a more 
% detailed comparison of the four cases regarding government intervention.
% Compile all_graphs.tex, using LaTeX, and produce (perhaps via .ps) a
% .pdf file, to see all graphs and their filenames.
% The graphs are in black and white, per the choice of
% BlackWhitePlots = 1; in SA_optim_plots.m  and
% BLACKWHITE = 1; in plots_2015.m
% Set these to = 0; to get color-lines, as in the working paper version.
%
% TABLES:
% The results in table 7 are provided as LaTeX-format tables at the
% towards the end of the output.  While one can use these tables directly
% in any ASCII-based LaTeX editor, one needs to re-arrange these a bit
% to obtain the exact format used in table 7.
% The tables resulting from running the program have been copied-and-pasted
% into all_graphs.tex, at the beginning, to see their results.
%
% PARAMETERS:
% The parameters chosen are listed in table 6 of the paper.  However, the
% numerical values there are reported only up to three digits.  However, a
% more exact version for the parameters q(0), nu,
% h-lowerbar and xi is needed to recreate the figures. These exact values 
% are "hard-wired" and provided to the code below on line 400.  
% If you instead wish to re-optimize the choice for these parameters, 
% set DO_CALIBRATE = 1; below, rather than = 0;
% It may be wise to restart that re-optimization a few times, starting from
% the last value obtained. 

% This file used to be called driver_2014_10_10_mult_om.m

clear all; close all; clc

%% Initialize random number generator
randn('seed',123456)
rand('seed',123456)

%% Set parameters
% Labor Share ...dtheta_labshare ...  added 2014-09-29
% CYrat ... the ratio of cons to output ... moved here 2014-10-06
% Numerical parameter ... weight ... on "old solution", added 2014_10_06
% The average vplot_t = weight*vpmy_t + (1-weight)*vpsi_t,added 2014_10_06
global dsigma dphi domega dM_low dgamma ioutput dchi dtheta_labshare CYrat
% global vdy_t vpmy_t
global vpmy_t vdy_t va_t profits_t cons_e_t marg_v_e_t vpsi_t weight vplot_t
global FNpiece

vdy_arr = [];
vpmy_arr = [];
vpsi_arr = [];
vplot_arr = []; % = weight * vpmy_arr + (1-weight) vpsi_arr
                % = weight * "old and false solution" + (1-weight) * "new and correct solution"

% I eliminated the OLD_PARAMS option, 2014-04-15, Harald.

DO_CALIBRATE = 0; 
    % set to 1, if you seek to solve for the optimal parameters
    % set to 0, if you want to use existing parameters, eg for comparison 
    %    across no-risk-cases or different omegas.
DO_DEBUG = 0;

USE_NEW_SOLUTION = 1; 
    % set = 1, if you want to use the new, corrected solution, as of Sept 2014
    % set = 0, if you want to use the old, false solution before Sept 2014
Rbar_tenyears     = 1.04^10;
Qbar_tenyears     = 1.09^10; 

OMEGA_VALUE_1       = 0.1; 
OMEGA_VALUE_2       = 0.2; 

theta_1 = ((Qbar_tenyears/Rbar_tenyears) - 1)*(1-OMEGA_VALUE_1)/OMEGA_VALUE_1;
betta_tilde_1 = 1.0/(Rbar_tenyears * (1 - OMEGA_VALUE_1 + OMEGA_VALUE_1 * theta_1 ));

theta_2 = ((Qbar_tenyears/Rbar_tenyears) - 1)*(1-OMEGA_VALUE_2)/OMEGA_VALUE_2;
betta_tilde_2 = 1.0/(Rbar_tenyears * (1 - OMEGA_VALUE_2 + OMEGA_VALUE_2 * theta_2 ));

% riskprem_tenyears = 1.03^10; % = 1/0.63;
% Note that, in the paper, M_low is 1/riskprem_tenyears,
% whereas in the code, it appears that M_low = 1/(R*riskprem_tenyears)

if DO_CALIBRATE,
    GOVRISKCASES = 3;
else
    if DO_DEBUG,
        GOVRISKCASES = 3;
    else
        GOVRISKCASES = 1 : 4;
    end;
end;

for govriskcase = GOVRISKCASES,
    
    
    switch govriskcase
        case 1 % no gov. risk
            domega      = 0.0;
            % Risk preferences
            dM_low      = 1/Rbar_tenyears;
            dM_high     = 1/Rbar_tenyears; % should be the same as dM_low
            FNpiece = 'norisk';
        case 2 % no gov. risk premium
            domega      = OMEGA_VALUE_1;
            % Risk preferences
            dM_low      = 1/Rbar_tenyears;
            dM_high     = 1/Rbar_tenyears; % should be the same as dM_low
            FNpiece = 'noprem';
        case 3 % benchmark
            domega      = OMEGA_VALUE_1;
            % Risk preferences
            dM_low      = betta_tilde_1;
            dM_high     = theta_1*betta_tilde_1;
            FNpiece = 'model';
        case 4 % no gov. risk premium, omega-value 2
            domega      = OMEGA_VALUE_2;
            % Risk preferences
            dM_low      = 1/Rbar_tenyears;
            dM_high     = 1/Rbar_tenyears; % should be the same as dM_low
            FNpiece = 'nopremom2';
            % Note that 1 = E[ M Rbar_tenyears ] = ((1-domega)*dM_low + domega*dM_high)*Rbar_tenyears 
        otherwise
    end;


    
    % Government subsidy
    dsigma      = 0.5;
    
    % Government risk
    dphi        = 3;
    dphi_low    = 1;
    
    dchi        = 0.5;
    
    
    
    % Growth
    dgamma      = 1.35;
    ioutput     = 0;
    


    
    if DO_CALIBRATE,
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %
        %   STILL DOING SOME DIAGNOSTICS ON THIS:
        %
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Labor Share ... added 2014-09-29
        dtheta_labshare = 0.8; % 0.8;
        CYrat = 0.84; % 3/4; % to correct output for "missing investment"
        weight = 0; % weight on "old" solution, can be useful for homotopy approach
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
        %%  vX_opt = [ dxi, dh_bar, da_0, dnu ]; %
        

        % vX_Guess      = [0.15;65;25;0.4];
        
        % vX_Guess      = [ 0.7680; 0.8019; 4.7411; 0.4203 ]; % solution in prev version of paper
        
        % vX_Guess = [   0.1365; 28.0359; 9.3851; 0.4267 ];  % last value, crit = 0.0267
        % vX_Guess = [    0.5075; 1.7875; 4.1160; 0.4463 ];  % best value, crit = 0.0238
        
        
        % vX_Guess = [     0.3133; 8.8151; 8.6733; 0.4175 ]; % best value 2014-10-10B, crit = 1.2411e-07, OBJECTIVE = 1;

        % old values, when targeting shares of GDP net of investment:
        % vX_optA =         [ 0.3133; 8.8151; 8.6733; 0.4175]; % same as above
        % vX_optB =1.0e-04 *[ 0.1688; 0.2167; 0.4363; 0.0313]; % a very slight correction
        % vX_Guess = vX_optA + vX_optB;  % best value 2014-10-10C, crit = 4.6545e-06, OBJECTIVE = 2;

        vX_optA =         [  0.5380;  3.1350; 7.8308;  0.4133 ];  % same as below
        vX_optB =1.0e-04 *[ -0.2819; -0.2933; 0.4525; -0.4402 ]; % a very slight correction
        vX_Guess = vX_optA + vX_optB;  % best value 2015_05_28, crit = 4.9235e-06, OBJECTIVE = 2;
        
        
        vX_L          = [0;0;0;0];
        % vX_U          = [0.99;100;200;0.99];
        vX_U          = [0.99;200;200;0.99];
        vX_init      = vX_Guess;
        options     = optimset('MaxFunEvals',100000,'Display','off');
        

        SA_optim_val_old = 99;
        vX_opt = vX_init;
        vX_init_old = vX_init;

        CHOOSE_OPTIMIZER = 1;
                
        switch CHOOSE_OPTIMIZER,
            case 1
                % This optimizer uses fminsearch, the built-in MATLAB
                % routine, to find a minimum, but restarting the minimizer
                % RE_OPT_ITER times.  Additionally, there is a bit of
                % simulated annealing built in: if the next optimum is just
                % a bit worse than the previous one, jump there.  The
                % initial point is a random draw from the last best one.
                % Noise and annealing-treshold shrink over time, at
                % pre-specified speeds.
                
                RE_OPT_ITER = 200; % 1000; % 2000;
                % RE_OPT_ITER = 200;
                SA_optim_val_arr = zeros(1,RE_OPT_ITER);
                vX_opt_arr       = zeros(4,RE_OPT_ITER);
                flag_arr         = zeros(1,RE_OPT_ITER);
                
                % for a version of simulated annealing,using a treshold acceptance
                % version.  Parameters are always tricky.
                % Set "scale" to 0 to turn this off.
                cooling_speed = 0.01; % = 0; % keeps treshold constant
                cooling_scale = 0.01; % = 0; % gets rid of sim. ann.
                noise_scaleA  = 0.2;  % = 0; % add no noise to successful new optima for new starting point
                noise_scaleB  = 1;    % = 0; % bad idea --- no random search when stuck.
                noise_speed   = 0.005;% = 0; % keeps noise scale constant
                noise_sigma = diag([0.2,10,4,0.1]);
                tic
                for opt_iter_n = 1 : RE_OPT_ITER,
                    if opt_iter_n == floor(opt_iter_n/1000)*1000,
                        save results;
                    end;
                    vX_opt_old = vX_opt;
                    [vX_opt,SA_optim_val,iexitflag]  = fminsearch(@SA_optim_plots,vX_init,options);
                    totaltime = toc;
                    totalhrs = floor(totaltime/3600);
                    totalmin = floor(totaltime/60)-60*totalhrs;
                    totalsec = totaltime - 60*floor(totaltime/60);
                    timestr = sprintf('time %2.0f:%2.0f:%4.2f, ',totalhrs,totalmin,totalsec);
                    if SA_optim_val < SA_optim_val_old + cooling_scale*exp( - opt_iter_n * cooling_speed),
                        % noise_scaleA 
                        if SA_optim_val < SA_optim_val_old,
                            flag_arr(opt_iter_n) = 1;
                            disp([timestr,sprintf('iter. %3.0f out of %3.0f, improvement to %8.6f',opt_iter_n,RE_OPT_ITER,SA_optim_val)]);
                        elseif SA_optim_val > SA_optim_val_old + 0.000001,
                            flag_arr(opt_iter_n) = 3;
                            disp([timestr,sprintf('iter. %3.0f out of %3.0f, worsening per sim. ann.: %8.6f',opt_iter_n,RE_OPT_ITER,SA_optim_val)]);
                        else
                            flag_arr(opt_iter_n) = 2;
                            disp([timestr,sprintf('iter. %3.0f out of %3.0f, no improvement, but sim. ann.: %8.6f',opt_iter_n,RE_OPT_ITER,SA_optim_val)]);
                        end;
                        SA_optim_val_old = SA_optim_val;
                        noise_term = noise_sigma * randn(4,1);
                        noise_term = noise_scaleA * exp(-opt_iter_n * noise_speed) * noise_term;
                        vX_init = vX_opt + noise_term;
                        vX_init_old = vX_init;
                    else
                        % noise_scaleB
                        flag_arr(opt_iter_n) = 4;
                        disp([timestr,sprintf('iter. %3.0f out of %3.0f, no improvement.',opt_iter_n,RE_OPT_ITER)]);
                        vX_opt = vX_opt_old;
                        noise_term = noise_sigma * randn(4,1);
                        noise_term = noise_scaleB * exp(-opt_iter_n * noise_speed) * noise_term;
                        vX_init = vX_init_old + noise_term;
                        SA_optim_val = SA_optim_val_old;
                    end;
                    SA_optim_val_arr(opt_iter_n) = SA_optim_val;
                    vX_opt_arr(:,opt_iter_n) = vX_opt;
                end;
            case 2
                % My own version of constrained simulated annealing.
                % Given some optimum, take N_next_draws at random around it
                % and check the criterion there.  Take the best one of
                % these.
                % If the next optimum is just
                % a bit worse than the previous one, jump there.  The
                % initial point is a random draw from the last best one.
                % Noise and annealing-treshold shrink over time, at
                % pre-specified speeds.
                
                RE_OPT_ITER = 20000;
                % RE_OPT_ITER = 200;
                SA_optim_val_arr = zeros(1,RE_OPT_ITER);
                vX_opt_arr       = zeros(4,RE_OPT_ITER);
                flag_arr         = zeros(1,RE_OPT_ITER);
                
                N_next_draws = 600;
                
                N_next_draws_rescale1 = floor(N_next_draws/2);
                N_next_draws_rescale2 = floor(N_next_draws*3/4);
                vX_draws = zeros(4,N_next_draws);
                df_draws = zeros(1,N_next_draws);
                
                % for a version of simulated annealing,using a treshold acceptance
                % version.  Parameters are always tricky.
                % Set "scale" to 0 to turn this off.
                cooling_speed = 0.01; % = 0; % keeps treshold constant
                cooling_scale = 0.01; % = 0; % gets rid of sim. ann.
                noise_scaleA  = 0.2;  % = 0; % add no noise to successful new optima for new starting point
                noise_scaleB  = 1;    % = 0; % bad idea --- no random search when stuck.
                noise_scaleC  = 0.5;  % = 0; % bad idea --- all N_next_draws will be the same.
                noise_rescaleC1 = 1;
                noise_rescaleC2 = 1;
                rescale_factor = 1.1;
                noise_speed   = 0.005;% = 0; % keeps noise scale constant
                noise_sigma = diag([0.5,5,1,0.1]);

                tic
                for opt_iter_n = 1 : RE_OPT_ITER,
                    if opt_iter_n == floor(opt_iter_n/1000)*1000,
                        save results;
                    end;
                    vX_opt_old = vX_opt;
                    jmin = 0;
                    SA_optim_val = 1000000000000;
                    for j = 1 : N_next_draws, % noise_scaleC  
                        % create draw:
                        noise_term = noise_sigma * randn(4,1);
                        noise_term = noise_scaleC * exp(-opt_iter_n * noise_speed) * noise_term;
                        if j > N_next_draws_rescale1,
                            noise_term = noise_rescaleC1*noise_term; 
                            if j > N_next_draws_rescale2,
                                noise_term = noise_rescaleC2*noise_term;
                            end;
                        end;
                        vX_draws(:,j) = vX_init + noise_term;  
                        % but redo for xi, the first entry, to constrain it
                        % between 0 and 1: do a tan-arctan transformation:
                        vX_draws(1,j) = (atan(  tan( pi*(vX_init(1)-0.5)) + noise_term(1) ))/pi + 0.5;
                        % and redo for a, the third entry, to constraint it
                        % to be positive, per log-exp transformation
                        vX_draws(3,j) = exp( log(vX_init(3)) + noise_term(3) );
                        % test draw:
                        df_draws(j) = SA_optim_plots( vX_draws(:,j) );
                        if df_draws(j) < SA_optim_val,
                            SA_optim_val = df_draws(j);
                            vX_opt_new = vX_draws(:,j);
                            jmin = j;
                        end;
                    end;
                    totaltime = toc;
                    totalhrs = floor(totaltime/3600);
                    totalmin = floor(totaltime/60)-60*totalhrs;
                    totalsec = totaltime - 60*floor(totaltime/60);
                    timestr = sprintf('time %2.0f:%2.0f:%4.2f, ',totalhrs,totalmin,totalsec);
                    if SA_optim_val < SA_optim_val_old + cooling_scale*exp( - opt_iter_n * cooling_speed),
                        % noise_scaleA  
                        if SA_optim_val < SA_optim_val_old,
                            flag_arr(opt_iter_n) = 1;
                            disp([timestr,sprintf('iter. %3.0f out of %3.0f, improvement to %8.6f',opt_iter_n,RE_OPT_ITER,SA_optim_val)]);
                        elseif SA_optim_val > SA_optim_val_old + 0.000001,
                            flag_arr(opt_iter_n) = 3;
                            disp([timestr,sprintf('iter. %3.0f out of %3.0f, worsening per sim. ann.: %8.6f',opt_iter_n,RE_OPT_ITER,SA_optim_val)]);
                        else
                            flag_arr(opt_iter_n) = 2;
                            disp([timestr,sprintf('iter. %3.0f out of %3.0f, no improvement, but sim. ann.: %8.6f',opt_iter_n,RE_OPT_ITER,SA_optim_val)]);
                        end;
                        SA_optim_val_old = SA_optim_val;
                        vX_opt = vX_opt_new; 
                        vX_opt_old = vX_opt_new;
                        noise_term = noise_sigma * randn(4,1);
                        noise_term = noise_scaleA * exp(-opt_iter_n * noise_speed) * noise_term;
                        vX_init = vX_opt + noise_term;
                        % but redo for xi, the first entry, to constrain it
                        % between 0 and 1: do a tan-arctan transformation:
                        vX_init(1) = (atan(  tan( pi*(vX_opt(1)-0.5)) + noise_term(1) ))/pi + 0.5;
                        % and redo for a, the third entry, to constraint it
                        % to be positive, per log-exp transformation
                        vX_init(3) = exp( log(vX_opt(3)) + noise_term(3) );
                        vX_init_old = vX_init;
                        noise_rescaleC1 = 1;
                        noise_rescaleC2 = 1;
                    else
                        % noise_scaleB  
                        flag_arr(opt_iter_n) = 4;
                        disp([timestr,sprintf('iter. %3.0f out of %3.0f, no improvement.',opt_iter_n,RE_OPT_ITER)]);
                        vX_opt = vX_opt_old;
                        noise_term = noise_sigma * randn(4,1);
                        noise_term = noise_scaleB * exp(-opt_iter_n * noise_speed) * noise_term;
                        vX_init = vX_init_old + noise_term;
                        % but redo for xi, the first entry, to constrain it
                        % between 0 and 1: do a tan-arctan transformation:
                        vX_init(1) = (atan(  tan( pi*(vX_init_old(1)-0.5)) + noise_term(1) ))/pi + 0.5;
                        % and redo for a, the third entry, to constraint it
                        % to be positive, per log-exp transformation
                        vX_init(3) = exp( log(vX_init_old(3)) + noise_term(3) );
                        SA_optim_val = SA_optim_val_old;
                        noise_rescaleC1 = rescale_factor*noise_rescaleC1;
                        noise_rescaleC2 = rescale_factor*noise_rescaleC2;
                    end;
                    SA_optim_val_arr(opt_iter_n) = SA_optim_val;
                    vX_opt_arr(:,opt_iter_n) = vX_opt;
                end;
            otherwise
               disp('**** check code driver_2014_09_29_mult_om: invalid CHOOSE_OPTIMIZER');
               disp('**** Terminating the calculations');
               return;
        end;
        
        vX_opt_last = vX_opt;
        SA_optim_val_last = SA_optim_val;
        [SA_optim_val,opt_indx]=min(SA_optim_val_arr);
        vX_opt = vX_opt_arr(:,opt_indx);
         
        disp('optimization criterion, last value:');
        disp(SA_optim_val_last);
        disp(['optimization criterion, best value (at ',sprintf('%3.0f out of %3.0f):',opt_indx,RE_OPT_ITER)]);
        disp(SA_optim_val);
        
        disp('last vX:');
        disp(vX_opt_last);
        disp('optimal vX:'); 
        disp(vX_opt);

        
        save vX_opt.mat vX_opt SA_optim_val vX_opt_last SA_optim_val_last
        % save vX_opt.mat vX_opt SA_optim_val
        %     optim_val = SA_optim_plots(vX_init);
        
        
        disp(['Time taken for optimization: ',sprintf('%2.0f:%2.0f:%4.2f.',totalhrs,totalmin,totalsec)]);
        
        save results
    else
        if USE_NEW_SOLUTION,
            % Labor Share ... added 2014-09-29
            dtheta_labshare = 0.8; % 0.8;
            CYrat = 0.84; % 3/4; % to correct output for "missing investment"
            weight = 0; % weight on "old" solution, can be useful for homotopy approach
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % load vX_opt
            %%  vX_opt = [ dxi, dh_bar, da_0, dnu ]; %
            
            % vX_opt = [     0.3133; 8.8151; 8.6733; 0.4175 ]; % best value 2014-10-10B, crit = 1.2411e-07, OBJECTIVE = 1;
 
            % % For old targets: shares of GDP net of investment
            % vX_optA =         [ 0.3133; 8.8151; 8.6733; 0.4175]; % same as above
            % vX_optB =1.0e-04 *[ 0.1688; 0.2167; 0.4363; 0.0313]; % a very slight correction
            % vX_opt = vX_optA + vX_optB;  % best value 2014-10-10C, crit = 4.6545e-06, OBJECTIVE = 2;

            vX_optA =         [  0.5380;  3.1350; 7.8308;  0.4133 ];  % same as above
            vX_optB =1.0e-04 *[ -0.2819; -0.2933; 0.4525; -0.4402 ]; % a very slight correction
            vX_opt  = vX_optA + vX_optB;  % best value 2015_05_28, crit = 4.9235e-06, OBJECTIVE = 2;
                  
                     
        else % old solution
            weight = 1;
            dtheta_labshare = 1;
            CYrat = 1;
            %% For old version, with theta_labshare = 0; CYrat = 1; using phi rather than psi:
            vX_opt = [ 0.7680; 0.8019; 4.7411; 0.4203 ]; % Before 2014_09_29. Reached after RESTARTS = 10 from vX_Guess2, which converged
            % vX_opt = [ 0.7783; 1.1125; 7.0281; 0.3968 ]; % Before 2014_09_29. Best so far for dsigma = 0.5, dphi = 3, domega = 0.1
        end;
    end;
    
    ioutput = 1;
    SA_optim_plots(vX_opt);
    
    vdy_arr = [ vdy_arr,vdy_t];
    vpmy_arr = [ vpmy_arr, vpmy_t];
    vpsi_arr = [ vpsi_arr, vpsi_t]; 
    vplot_arr = [ vplot_arr, vplot_t];    
end;

if (~DO_CALIBRATE) & (~DO_DEBUG),
    plots_2015;
    % I re-activated this line, Harald 2014-04-16
    % I added the following lines which are useful for manually updating
    % the table regarding the model results in the paper
    timeline = 1960:10:1960+20*10;
    disp('Health shares in percent, ordered benchmark -- no gov. risk prem,om2 -- no gov. risk prem -- no risk prem.:');
    % Before 2014_09_29:
    % dispmat([timeline',vpmy_arr(:,[3,4,2,1])*100],1,1,1);
    % ... but it should be share of total output, so
    disp(' ');
    dispmat([timeline',vpsi_arr(:,[3,4,2,1])*100],1,1,1);
    disp(' ');
    disp('\newpage');
    disp('R\&D shares in percent, ordered benchmark -- no gov. risk prem,om2 -- no risk prem.-- no gov. risk:');
    disp(' ');
    dispmat([timeline',vdy_arr(:,[3,4,2,1])*100],1,1,1);    
end;



